#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""

"""
from SLiCAP import *
#prj = initProject("Hearing Loop")
specFileName = "specifications.csv"

# Create a list with circuit file names if you want to run multiple circuits.
fileName = 'noisePropagation'

outputNoise = {}

# import the specifications
specs = csv2specs(specFileName)

i1 = instruction()
i1.setCircuit(fileName + '.cir')

# Define the circuit parameters from the specifications
specs2circuit(specs, i1)

# We will express the gain of the transadmitance in that of the total system
# and the output stage integrating voltage amplifier.

i1.setSimType('symbolic')
i1.setSource('V1')
i1.setDetector('V_L')
i1.setGainType('gain')
i1.setDataType("laplace")

gain = i1.execute()
V_o, V_i, B_T = sp.symbols('V_o, V_i, B_T')
BT = sp.solve(gain.laplace - V_o/V_i, B_T)[0]
i1.defPar('B_T', BT)

# We will define the source resistance 600 Ohm (should have been specified)
i1.defPar('R_g', 600)

i1.setSimType("numeric")
i1.setGainType('vi')
i1.setDataType('noise')

htmlPage('Numeric noise analysis')

head2html('Noise model')
text2html("Noise model hearing loop with frequency-independent " +
          "transadmittance amplifier and integrating voltage amplifier.")
img2html(fileName + '.svg', 1500, caption = "Noise model hearing loop system.")

text2html("The minimum output noise is obtained with all designable noise " +
          "contributions set to zero.")

S_iT, S_vT, S_iR, S_vR, S_B = sp.symbols('S_iT, S_vT, S_iR, S_vR, S_B')
i1.defPars({S_iT: 0, S_vT:0, S_iR: 0, S_vR: 0, S_B: 0})
noiseResult = i1.execute()

f_min = i1.getParValue('f_min')
f_max = i1.getParValue('f_max')
MinRMSnoise = rmsNoise(noiseResult, 'onoise', f_min, f_max)

text2html("It is determined by the source noise, receive coil noise, " +
          "and the receiver gain:")
eqn2html('V_n_Min', MinRMSnoise)
text2html("From this we obtain a minimum value $A_{Rmin}$ for $A_R$:")
sol = sp.solve(MinRMSnoise-i1.getParValue('v_on'), sp.Symbol("A_R"))
for solution in sol:
    if solution > 0:
        A_Rmin = solution
        eqn2html("A_Rmin", A_Rmin)
specs.append(
    specItem(
        "A_Rmin",
        description = "Minimum value receiver transmission coefficient $A$ (integrating receiver)",
        value    = A_Rmin,
        units       = "s",
        specType    = "design",
    )
)
# Save the specifications and display the specifications on a html page.
htmlPage("Updated specifications")
# Save the specifications
specs2csv(specs, "specifications.csv")
# Display them on an HTML page
specs2html(specs)